@@ -107,6 +107,32 @@
 #include <sys/fs/vx_ioctl.h>
 #endif
 
+#if defined(OSFV5) || defined(linux)
+#include <string.h>
+#endif
+
+#if defined(linux)
+#include <unistd.h>
+#include <stdio.h>
+#include <stdlib.h>
+#endif
+
+#if ((defined(solaris) && defined(__LP64__)) || defined(__s390x__))
+/* If we are building for 64-bit Solaris, all functions that return pointers
+ * must be declared before they are used; otherwise the compiler will assume
+ * that they return ints and the top 32 bits of the pointer will be lost,
+ * causing segmentation faults.  The following includes take care of this.
+ * It should be safe to add these for all other OSs too, but we're only
+ * doing it for Solaris now in case another OS turns out to be a special case.
+ */
+#include <stdio.h>
+#include <stdlib.h>
+#include <strings.h> /* For the BSD string functions */
+#endif
+
+void mbcopy(char *source, char *dest, size_t len);
+
+
 #if !defined(solaris) && !defined(off64_t) && !defined(_OFF64_T) && !defined(__off64_t_defined) && !defined(SCO_Unixware_gcc)
 typedef long long off64_t;
 #endif
@@ -122,11 +148,15 @@ extern int one;
  * cache, pointed to by async_init(gc) will be of
  * this structure type.
  */
-char version[] = "Libasync Version $Revision: 3.11 $";
+char version[] = "Libasync Version $Revision: 3.16 $";
 struct cache_ent {
 	struct aiocb myaiocb;			/* For use in small file mode */
 #ifdef _LARGEFILE64_SOURCE 
+#if defined(__CrayX1__)
+	aiocb64_t myaiocb64;		/* For use in large file mode */
+#else
 	struct aiocb64 myaiocb64;		/* For use in large file mode */
+#endif 
 #endif 
 	long long fd;				/* File descriptor */
 	long long size;				/* Size of the transfer */
@@ -175,6 +205,18 @@ void async_wait_for_write();
 void async_put_on_write_queue();
 void async_write_finish();
 
+/* On Solaris _LP64 will be defined by <sys/types.h> if we're compiling
+ * as a 64-bit binary.  Make sure that __LP64__ gets defined in this case,
+ * too -- it should be defined on the compiler command line, but let's
+ * not rely on this.
+ */
+#if defined(_LP64)
+#if !defined(__LP64__)
+#define __LP64__
+#endif
+#endif
+
+
 /***********************************************/
 /* Initialization routine to setup the library */
 /***********************************************/
@@ -261,9 +303,9 @@ long long depth;
 {
 	off64_t a_offset,r_offset;
 	long long a_size;
-	struct cache_ent *ce,*first_ce;
+	struct cache_ent *ce,*first_ce=0;
 	long long i;
-	ssize_t retval;
+	ssize_t retval=0;
 	ssize_t ret;
 	long long start = 0;
 	long long del_read=0;
@@ -273,7 +315,7 @@ long long depth;
 	/*
 	 * Check to see if it can be completed from the cache
 	 */
-	if(ce=(struct cache_ent *)incache(gc,fd,offset,size))
+	if((ce=(struct cache_ent *)incache(gc,fd,offset,size)))
 	{
 #ifdef _LARGEFILE64_SOURCE 
 #ifdef __LP64__
@@ -288,13 +330,20 @@ long long depth;
 			;
 		}
 		if(ret)
+		{
 			printf("aio_error 1: ret %d %d\n",ret,errno);
+		}
 #ifdef _LARGEFILE64_SOURCE 
 #ifdef __LP64__
 		retval=aio_return(&ce->myaiocb);
+#else
+#if defined(__CrayX1__)
+		retval=aio_return64((aiocb64_t *)&ce->myaiocb64);
 #else
 		retval=aio_return64((struct aiocb64 *)&ce->myaiocb64);
 #endif
+
+#endif
 #else
 		retval=aio_return(&ce->myaiocb);
 #endif
@@ -302,12 +351,12 @@ long long depth;
 		{
 #ifdef _LARGEFILE64_SOURCE 
 #ifdef __LP64__
-			mbcopy(ce->myaiocb.aio_buf,ubuffer,retval);
+			mbcopy((char *)ce->myaiocb.aio_buf,(char *)ubuffer,(size_t)retval);
 #else
-			mbcopy(ce->myaiocb64.aio_buf,ubuffer,retval);
+			mbcopy((char *)ce->myaiocb64.aio_buf,(char *)ubuffer,(size_t)retval);
 #endif
 #else
-			mbcopy(ce->myaiocb.aio_buf,ubuffer,retval);
+			mbcopy((char *)ce->myaiocb.aio_buf,(char *)ubuffer,(size_t)retval);
 #endif
 		}
 #ifdef _LARGEFILE64_SOURCE 
@@ -323,25 +372,25 @@ long long depth;
 			printf("aio_return error1: ret %d %d\n",retval,errno);
 #ifdef _LARGEFILE64_SOURCE 
 #ifdef __LP64__
-			printf("aio_return error1: fd %d offset %ld buffer %x size %d Opcode %d\n",
+			printf("aio_return error1: fd %d offset %ld buffer %lx size %d Opcode %d\n",
 				ce->myaiocb.aio_fildes,
 				ce->myaiocb.aio_offset,
-				ce->myaiocb.aio_buf,
+				(long)(ce->myaiocb.aio_buf),
 				ce->myaiocb.aio_nbytes,
 				ce->myaiocb.aio_lio_opcode
 #else
-			printf("aio_return error1: fd %d offset %lld buffer %x size %d Opcode %d\n",
+			printf("aio_return error1: fd %d offset %lld buffer %lx size %d Opcode %d\n",
 				ce->myaiocb64.aio_fildes,
 				ce->myaiocb64.aio_offset,
-				ce->myaiocb64.aio_buf,
+				(long)(ce->myaiocb64.aio_buf),
 				ce->myaiocb64.aio_nbytes,
 				ce->myaiocb64.aio_lio_opcode
 #endif
 #else
-			printf("aio_return error1: fd %d offset %d buffer %x size %d Opcode %d\n",
+			printf("aio_return error1: fd %d offset %d buffer %lx size %d Opcode %d\n",
 				ce->myaiocb.aio_fildes,
 				ce->myaiocb.aio_offset,
-				ce->myaiocb.aio_buf,
+				(long)(ce->myaiocb.aio_buf),
 				ce->myaiocb.aio_nbytes,
 				ce->myaiocb.aio_lio_opcode
 #endif
@@ -394,7 +443,7 @@ again:
 			continue;
 		if(r_offset+size > max)
 			continue;
-		if(ce=incache(gc,fd,r_offset,a_size))
+		if((ce=incache(gc,fd,r_offset,a_size)))
 			continue;
 		ce=alloc_cache(gc,fd,r_offset,a_size,(long long)LIO_READ);
 #ifdef _LARGEFILE64_SOURCE 
@@ -451,25 +500,25 @@ out:
 			printf("aio_return error2: ret %d %d\n",retval,errno);
 #ifdef _LARGEFILE64_SOURCE 
 #ifdef __LP64__
-			printf("aio_return error2: fd %d offset %lld buffer %x size %d Opcode %d\n",
+			printf("aio_return error2: fd %d offset %lld buffer %lx size %d Opcode %d\n",
 				first_ce->myaiocb.aio_fildes,
 				first_ce->myaiocb.aio_offset,
-				first_ce->myaiocb.aio_buf,
+				(long)(first_ce->myaiocb.aio_buf),
 				first_ce->myaiocb.aio_nbytes,
 				first_ce->myaiocb.aio_lio_opcode
 #else
-			printf("aio_return error2: fd %d offset %lld buffer %x size %d Opcode %d\n",
+			printf("aio_return error2: fd %d offset %lld buffer %lx size %d Opcode %d\n",
 				first_ce->myaiocb64.aio_fildes,
 				first_ce->myaiocb64.aio_offset,
-				first_ce->myaiocb64.aio_buf,
+				(long)(first_ce->myaiocb64.aio_buf),
 				first_ce->myaiocb64.aio_nbytes,
 				first_ce->myaiocb64.aio_lio_opcode
 #endif
 #else
-			printf("aio_return error2: fd %d offset %d buffer %x size %d Opcode %d\n",
+			printf("aio_return error2: fd %d offset %d buffer %lx size %d Opcode %d\n",
 				first_ce->myaiocb.aio_fildes,
 				first_ce->myaiocb.aio_offset,
-				first_ce->myaiocb.aio_buf,
+				(long)(first_ce->myaiocb.aio_buf),
 				first_ce->myaiocb.aio_nbytes,
 				first_ce->myaiocb.aio_lio_opcode
 #endif
@@ -479,12 +528,12 @@ out:
 		{
 #ifdef _LARGEFILE64_SOURCE 
 #ifdef __LP64__
-			mbcopy(first_ce->myaiocb.aio_buf,ubuffer,retval);
+			mbcopy((char *)first_ce->myaiocb.aio_buf,(char *)ubuffer,(size_t)retval);
 #else
-			mbcopy(first_ce->myaiocb64.aio_buf,ubuffer,retval);
+			mbcopy((char *)first_ce->myaiocb64.aio_buf,(char *)ubuffer,(size_t)retval);
 #endif
 #else
-			mbcopy(first_ce->myaiocb.aio_buf,ubuffer,retval);
+			mbcopy((char *)first_ce->myaiocb.aio_buf,(char *)ubuffer,(size_t)retval);
 #endif
 		}
 		first_ce->direct=0;
@@ -509,7 +558,7 @@ struct cache *gc;
 long long fd,size,op;
 off64_t offset;
 {
-	struct cache_ent *ce,*move;
+	struct cache_ent *ce;
 	long temp;
 	ce=(struct cache_ent *)malloc((size_t)sizeof(struct cache_ent));
 	if(ce == (struct cache_ent *)0)
@@ -759,9 +808,9 @@ long long depth;
 {
 	off64_t a_offset,r_offset;
 	long long a_size;
-	struct cache_ent *ce,*first_ce;
+	struct cache_ent *ce,*first_ce=0;
 	long long i;
-	ssize_t retval;
+	ssize_t retval=0;
 	ssize_t ret;
 	long long del_read=0;
 	long long start=0;
@@ -771,7 +820,7 @@ long long depth;
 	/*
 	 * Check to see if it can be completed from the cache
 	 */
-	if(ce=(struct cache_ent *)incache(gc,fd,offset,size))
+	if((ce=(struct cache_ent *)incache(gc,fd,offset,size)))
 	{
 #ifdef _LARGEFILE64_SOURCE 
 #ifdef __LP64__
@@ -839,25 +888,25 @@ long long depth;
 			printf("aio_return error4: ret %d %d\n",retval,errno);
 #ifdef _LARGEFILE64_SOURCE 
 #ifdef __LP64__
-			printf("aio_return error4: fd %d offset %lld buffer %x size %d Opcode %d\n",
+			printf("aio_return error4: fd %d offset %lld buffer %lx size %d Opcode %d\n",
 				ce->myaiocb.aio_fildes,
 				ce->myaiocb.aio_offset,
-				ce->myaiocb.aio_buf,
+				(long)(ce->myaiocb.aio_buf),
 				ce->myaiocb.aio_nbytes,
 				ce->myaiocb.aio_lio_opcode
 #else
-			printf("aio_return error4: fd %d offset %lld buffer %x size %d Opcode %d\n",
+			printf("aio_return error4: fd %d offset %lld buffer %lx size %d Opcode %d\n",
 				ce->myaiocb64.aio_fildes,
 				ce->myaiocb64.aio_offset,
-				ce->myaiocb64.aio_buf,
+				(long)(ce->myaiocb64.aio_buf),
 				ce->myaiocb64.aio_nbytes,
 				ce->myaiocb64.aio_lio_opcode
 #endif
 #else
-			printf("aio_return error4: fd %d offset %d buffer %x size %d Opcode %d\n",
+			printf("aio_return error4: fd %d offset %d buffer %lx size %d Opcode %d\n",
 				ce->myaiocb.aio_fildes,
 				ce->myaiocb.aio_offset,
-				ce->myaiocb.aio_buf,
+				(long)(ce->myaiocb.aio_buf),
 				ce->myaiocb.aio_nbytes,
 				ce->myaiocb.aio_lio_opcode
 #endif
@@ -921,7 +970,7 @@ again:
 			continue;
 		if(r_offset+size > max)
 			continue;
-		if(ce=incache(gc,fd,r_offset,a_size))
+		if((ce=incache(gc,fd,r_offset,a_size)))
 			continue;
 		ce=alloc_cache(gc,fd,r_offset,a_size,(long long)LIO_READ);
 #ifdef _LARGEFILE64_SOURCE 
@@ -999,25 +1048,25 @@ out:
 			printf("aio_return error5: ret %d %d\n",retval,errno);
 #ifdef _LARGEFILE64_SOURCE 
 #ifdef __LP64__
-			printf("aio_return error5: fd %lld offset %d buffer %x size %d Opcode %d\n",
+			printf("aio_return error5: fd %d offset %lld buffer %lx size %d Opcode %d\n",
 				first_ce->myaiocb.aio_fildes,
 				first_ce->myaiocb.aio_offset,
-				first_ce->myaiocb.aio_buf,
+				(long)(first_ce->myaiocb.aio_buf),
 				first_ce->myaiocb.aio_nbytes,
 				first_ce->myaiocb.aio_lio_opcode
 #else
-			printf("aio_return error5: fd %lld offset %d buffer %x size %d Opcode %d\n",
+			printf("aio_return error5: fd %d offset %lld buffer %lx size %d Opcode %d\n",
 				first_ce->myaiocb64.aio_fildes,
 				first_ce->myaiocb64.aio_offset,
-				first_ce->myaiocb64.aio_buf,
+				(long)(first_ce->myaiocb64.aio_buf),
 				first_ce->myaiocb64.aio_nbytes,
 				first_ce->myaiocb64.aio_lio_opcode
 #endif
 #else
-			printf("aio_return error5: fd %d offset %d buffer %x size %d Opcode %d\n",
+			printf("aio_return error5: fd %d offset %ld buffer %lx size %d Opcode %d\n",
 				first_ce->myaiocb.aio_fildes,
 				first_ce->myaiocb.aio_offset,
-				first_ce->myaiocb.aio_buf,
+				(long)(first_ce->myaiocb.aio_buf),
 				first_ce->myaiocb.aio_nbytes,
 				first_ce->myaiocb.aio_lio_opcode
 #endif
@@ -1040,7 +1089,6 @@ out:
 		takeoff_cache(gc,first_ce);
 		putoninuse(gc,first_ce);
 	}
-out2:
 	return((int)retval);	
 }
 
@@ -1167,10 +1215,7 @@ again:
 			exit(177);
 		}
 	} 
-	else	
-	{
-		return((ssize_t)size);
-	}
+	return((ssize_t)size);
 }
 
 /*************************************************************************
@@ -1188,7 +1233,7 @@ long long w_depth;
 long long direct;
 char *buffer,*free_addr;
 {
-	struct cache_ent *ce,*move;
+	struct cache_ent *ce;
 	long temp;
 	if(fd==0LL)
 	{
@@ -1377,9 +1422,14 @@ struct cache *gc;
 #ifdef _LARGEFILE64_SOURCE 
 #ifdef __LP64__
 	retval=aio_return(&ce->myaiocb);
+#else
+#if defined(__CrayX1__)
+	retval=aio_return64((aiocb64_t *)&ce->myaiocb64);
 #else
 	retval=aio_return64((struct aiocb64 *)&ce->myaiocb64);
 #endif
+
+#endif
 #else
 	retval=aio_return(&ce->myaiocb);
 #endif
@@ -1454,22 +1504,22 @@ again:
 			printf("Error in aio_write: ret %d errno %d\n",ret,errno);
 #ifdef _LARGEFILE64_SOURCE 
 #ifdef __LP64__
-			printf("aio_write_no_copy: fd %d buffer %x offset %lld size %d\n",
+			printf("aio_write_no_copy: fd %d buffer %lx offset %lld size %d\n",
 				ce->myaiocb.aio_fildes,
-				ce->myaiocb.aio_buf,
+				(long)(ce->myaiocb.aio_buf),
 				ce->myaiocb.aio_offset,
 				ce->myaiocb.aio_nbytes);
 #else
-			printf("aio_write_no_copy: fd %d buffer %x offset %lld size %d\n",
+			printf("aio_write_no_copy: fd %d buffer %lx offset %lld size %d\n",
 				ce->myaiocb64.aio_fildes,
-				ce->myaiocb64.aio_buf,
+				(long)(ce->myaiocb64.aio_buf),
 				ce->myaiocb64.aio_offset,
 				ce->myaiocb64.aio_nbytes);
 #endif
 #else
-			printf("aio_write_no_copy: fd %d buffer %x offset %lld size %d\n",
+			printf("aio_write_no_copy: fd %d buffer %lx offset %ld size %d\n",
 				ce->myaiocb.aio_fildes,
-				ce->myaiocb.aio_buf,
+				(long)(ce->myaiocb.aio_buf),
 				ce->myaiocb.aio_offset,
 				ce->myaiocb.aio_nbytes);
 #endif
@@ -1482,9 +1532,9 @@ again:
 	}
 }
 
-mbcopy(source, dest, len)
+void mbcopy(source, dest, len)
 char *source,*dest;
-int len;
+size_t len;
 {
 	int i;
 	for(i=0;i<len;i++)
