@@ -20,10 +20,31 @@
 #include <sys/fcntl.h>
 #endif
 
-#if defined(OSFV5)
+#if defined(OSFV5) || defined(linux)
 #include <string.h>
 #endif
 
+#if defined(linux)
+#include <unistd.h>
+#include <stdlib.h>
+#endif
+
+#if ((defined(solaris) && defined( __LP64__ )) || defined(__s390x__))
+/* If we are building for 64-bit Solaris, all functions that return pointers
+ * must be declared before they are used; otherwise the compiler will assume
+ * that they return ints and the top 32 bits of the pointer will be lost,
+ * causing segmentation faults.  The following includes take care of this.
+ * It should be safe to add these for all other OSs too, but we're only
+ * doing it for Solaris now in case another OS turns out to be a special case.
+ */
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <unistd.h>
+#include <stdlib.h>
+#include <string.h>
+#endif
+
+
 #ifdef HAVE_ANSIC_C
 /************************************************************************/
 /* Here is the API... Enjoy 						*/
@@ -57,7 +78,7 @@ void do_label(int,char *,int,int);
 /*	  column							*/
 /************************************************************************/
 
-char libbif_version[] = "Libbif Version $Revision: 3.5 $";
+char libbif_version[] = "Libbif Version $Revision: 3.11 $";
 void do_eof(int );		/* Used internally */
 void do_header(int );		/* Used internally */
 #endif
@@ -240,7 +261,6 @@ int row,column;
 #endif
 	struct float_record floatrec;
 	short s_row,s_column;
-	int i;
 	unsigned char *sptr,*dptr;
 	s_row=(short)row;
 	s_column=(short)column;
@@ -253,11 +273,12 @@ int row,column;
         floatrec.rgblo=0x0;
         floatrec.hi_row=(char)(s_row&0xff);
         floatrec.lo_row=(char)((s_row>>8)&0xff);
-        floatrec.hi_column=(char)(column&0xff);
-        floatrec.lo_column=(char)((column>>8)&0xff);
+        floatrec.hi_column=(char)(s_column&0xff);
+        floatrec.lo_column=(char)((s_column>>8)&0xff);
 	sptr =(unsigned char *) &value;
 	dptr =(unsigned char *) &floatrec.data;
-#if defined(BIG_ENDIAN) && !defined(linux)
+/*#if defined(BIG_ENDIAN) && !defined(linux)*/
+#if defined(ZBIG_ENDIAN) 
 	dptr[0]=sptr[7]; /* Convert to Little Endian */
 	dptr[1]=sptr[6];
 	dptr[2]=sptr[5];
